package com.jeffknecht.dynadao.annotation;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * <p>Annotates a parameter name with the property
 * name of the DTO for DynaDaoProxy implementations.</p>
 * 
 * <p>Note that if you are supplying the same Parameter
 * more than once in a method signature, you need to 
 * specify a different placeholder value for each
 * copy that parameter, otherwise the same value
 * will be used for each instance of that parameter
 * when evaluating the query</p>
 * 
 * @author jeff.knecht
 *
 */
@Target(ElementType.PARAMETER)
@Retention(RetentionPolicy.RUNTIME)
public @interface Parameter {
	
	/**
	 * Support comparison operators.
	 * 
	 * @author jeff.knecht
	 */
	public static enum Operator {
		EQUALS {
			public String sql() {
				return "=";
			}
		},
		NOT_EQUALS {
			public String sql() {
				return "<>";
			}
		},
		GREATER_THAN {
			public String sql() {
				return ">";
			}
		},
		LESS_THAN {
			public String sql() {
				return "<";
			}
		},
		LIKE {
			public String sql() {
				return "like";
			}
		},
		IN {
			public String sql() {
				return "in";
			}
		};
		
		public abstract String sql();
	}

	/**
	 * The name of the attribute on the DTO
	 * referenced by this parameter.
	 * 
	 * @return
	 */
	public String value();
	
	/**
	 * The name to use for the placeholder in the
	 * generated where clause.  If not supplied,
	 * the where clause generator will use this
	 * annotations value() property.
	 * 
	 * @return
	 */
	public String placeholder() default "";

	/**
	 * How to compare the values in the generated
	 * where clause. Default is EQUALS.
	 * 
	 * @return
	 */
	public Operator comparison() default Operator.EQUALS;
	
	/**
	 * Controls whether string comparisons are performed in a case-insensitive manner.
	 * Default is false (case-sensitive).
	 * 
	 * @return
	 */
	public boolean caseInsensitive() default false;
}
